/*global
  window:false
  document:false
  console:false
  disableSelection:false
  Image:false
  CONFIG:false
  Button:true
*/

/*
  --------------------------------------------------------------------
  helper functions
  --------------------------------------------------------------------
*/

function _disabled(event) {
    "use strict";
    event.stopPropagation();
    event.preventDefault();
    return false;
}

function _set_geometry(x, geometry) {
    "use strict";
    // set geometry
    x.style.left = geometry[0] + 'px';
    x.style.top = geometry[1] + 'px';
    x.style.width = geometry[2] + 'px';
    x.style.height = geometry[3] + 'px';
}

function _set_attributes(x) {
    if ( ! CONFIG.selectable) {
	disableSelection(x);
    }
    if ( ! CONFIG.cursor ) {
        x.style.cursor = "none";
    }
    if (CONFIG.touch) {
	x.ontouchstart = _disabled;
	x.ontouchmove = _disabled;
	x.ontouchend = _disabled;
	x.ontouchenter = _disabled;
	x.ontouchleave = _disabled;
    }
}

function _create_div(parent, geometry) {
    "use strict";
    var x = document.createElement('div');
    parent.appendChild(x);
    _set_geometry(x, geometry);
    return x;
}

function _create_img(parent, geometry) {
    "use strict";
    var x = document.createElement('img');
    parent.appendChild(x);
    _set_geometry(x, geometry);
    return x;
}

//=========================================================
// Button
//=========================================================
Button = function(parent, data, geometry, background_color, border_color) {
    "use strict";
    var x = _create_img(parent, geometry);
    _set_attributes(x);

    x.style.position = 'absolute';
    x.style.fontSize = '11px';
    x.style.fontFamily = 'Verdana, Arial, Helvetica, sans-serif';
    x.innerHTML = data.label;
    x.style.zIndex = 1;
    x.style.margin = 0 + 'px';
    x.style.padding = 0 + 'px';
    //x.style.paddingLeft = 12 + 'px';
    //x.style.cursor = 'pointer';
    x.style.color = 'rgba(250,250,250,0.8)';
    //x.hover = false;
    x.selected = false;
    disableSelection(x);


    // preload images: prevent flashing buttons caused by loading image on the fly
    for (var k = 0; k < data.url.length; ++k) {
        var image = new Image();
        image.src = data.url[k];
    }

    x.src = data.url[0];

    var p = data.padding;
    x.style.padding = p + "px " + p + "px " + p + "px " + p + "px";
    x.style.margin = "8px 8px 8px 8px";
    x.style.borderStyle = "solid";
    x.style.borderWidth = "4px";
    x.style.borderColor = border_color;
    x.style.borderRadius = "20px";
    x.style.boxSizing = "border-box";
    x.style.MozBoxSizing = "border-box";

    if (CONFIG.mouse) {
        x.onclick = data.callback;
    }
    if (CONFIG.touch) {
        x.ontouchstart = data.callback;
    }

    this.button = x;
    this.background_color = background_color;

    this.url = data.url;
    this.counter = 0;
};

Button.prototype.set_geometry = function(geometry) {
    _set_geometry(this.button, geometry);
};

Button.prototype.set = function(state, hover) {
    "use strict";
    this.button.selected = state;
    this.button.style.backgroundColor =
        this.background_color[hover?1:0][this.button.selected ? 1 : 0];
};

Button.prototype.increment = function() {
    "use strict";
    this.counter = (this.counter + 1) % this.url.length;
    var c = this.counter;
    this.button.src = this.url[c];
    this.set(c !== 0, false);
};


//=========================================================
// ButtonBox
//=========================================================
ButtonBox = function(touchcontrols, parent, data_array, geometry, button_background_color, button_border_color) {
    "use strict";

    var count = data_array.length;
    var size = geometry[3] / count;
    var button_size = size - 16;

    var x = _create_div(parent, geometry);
    _set_attributes(x);

    x.style.position = 'absolute';

    x.style.fontSize = '10px';
    x.style.fontFamily = 'Verdana, Arial, Helvetica, sans-serif';
    x.style.color = 'rgba(255,255,255,1.0)';
    x.style.padding = "0px 0px 0px 0px";
    x.style.margin = "0px 0px 0px 0px";
    x.style.border = "0px 0px 0px 0px";

    x.style.backgroundColor = 'rgba(50,50,50,0.0)';

    this.box = x;

    // create buttons
    this.button = [];
    count = 0;
    var y = 0;

    for (var i = 0; i < data_array.length; ++i) {

        var data = data_array[i];
        geometry = [0, y, button_size, button_size];

        this.button[count] = new Button( this.box, data, geometry, button_background_color, button_border_color );

        var b = this.button[count];

        ++count;
        y += size;
    }
};

ButtonBox.prototype.set_geometry = function(xpos) {
    "use strict";

    var window_height = document.body.clientHeight;
    var window_width = document.body.clientWidth;
    var count = 6;
    var size = window_height / count;
    var button_size = window_height / count - 16;

    // box
    var geometry = [0, 0, size, size*count];

    _set_geometry(this.box, geometry);

    // buttons
    for (var i = 0; i < count; ++i) {
        geometry = [xpos, i*size, button_size, button_size];
        var b = this.button[i];
        b.set_geometry(geometry);
    }
};
