/*global
  XLAB:false
  console:false
  document:false
  window:false
  xmat4:false
  Multishader:false
  load_surface:false
  clone:false
  COLORMODE_TEXIMAGE
  COLORMODE_CUBEMAP
  TEXIMAGE_TEXTURE_UNIT
  CUBEMAP_TEXTURE_UNIT
  disableSelection:false
  CONFIG:false
  surface_data:false
  parse_int:false
  format_sci:false
  Color:false
  TouchControls:false
*/

/*
  Base URL

  1. Gets the URL of this file, which is for example
  http://localhost:8000/js/viewer.js
  or
  file:///home/nick/work/webgl/touchviewer/js/viewer.js
  2. Remove last two entries, to become
  http://localhost:8000/
  or
  file:///home/nick/work/webgl/touchviewer/
*/
function getBaseURL() {
    "use strict";
    var s = document.getElementsByTagName("script");
    var b = s[s.length-1].src;
    b = b.split('/');
    b = b.slice(0, b.length-2);
    var url = b.join('/') + '/';
    return url;
}
var baseURL = getBaseURL();

Viewer = function() {
    "use strict";
    this.gl_configuration =
        {
            'antialias' : CONFIG.opengl_antialias,
            'debug' : CONFIG.opengl_debug
        };

    //this.background_color = [ 132.0/255.0, 32.0/255.0, 35.0/255.0];
    this.background_color = CONFIG.canvas_color;
    this.pline_speed = [0.001, 0.001];

    this.initial_surface = CONFIG.initial_surface;

    this.init();
};


Viewer.prototype.init = function() {
    "use strict";

    // disable selectable
    if ( ! CONFIG.selectable ) {
        disableSelection(document.body);
    }

    // disable draggable
    if ( ! CONFIG.draggable ) {
        window.draggable = false; // only supported on some browsers
        window.ondragstart = function() { return false; };
    }

    // disable context menu (right click)
    if ( ! CONFIG.contextmenu ) {
        window.oncontextmenu = function() { return false; };
    }

    // body
    this._setup_body();

    // window resize callback
    window.onresize = this.onResize;


    this.viewer = new XLAB.Viewer(document.body, this.gl_configuration);
    this.canvas = this.viewer.canvas;


    // disable cursor
    if ( ! CONFIG.cursor ) {
        document.body.style.cursor = "none";
        this.canvas.style.cursor = "none";
    }
    if ( ! CONFIG.selectable ) {
	disableSelection(this.canvas);
    }

    this.gl = this.viewer.gl;
    
    this.extensions =
        [
            this.gl.getExtension("OES_standard_derivatives")
        ];

    var b = this.background_color;
    this.gl.clearColor(b[0], b[1], b[2], 1.0);
    this.gl.enable(this.gl.DEPTH_TEST);

    this.animator = new XLAB.Animator();

    if(CONFIG.mouse) {
        this.mouse = new XLAB.Mouse(this.animator,this.canvas);
    }
    if(CONFIG.mousewheel) {
        this.mousewheel = new XLAB.MouseWheel(this.animator);
    }
    if(CONFIG.touch) {
	this.touch = new XLAB.Touch(this.animator,this.canvas);
    }


    this.pline_position = [0.0, 0.0];
    this.surface = [];
    this.surface_index = 0;

    // initialize position
    this.model_matrix = xmat4.create();
    this.surface_part = 2;

    this.load_teximage();
    this.load_cubemap();

    this.load_shader();

    this.load_controls();

    this.camera = new XLAB.Camera();
    if ( CONFIG.fps_counter ) {
        this.fpscounter = new XLAB.FPSCounter();
    }

    this.color = new Color(this);

    this._load_surfaces();

    this.viewer.run(
        function(this0) {
            return function(gl) { this0.draw(gl); };
        }(this)
    );
};

Viewer.prototype._set_camera = function(cam_transform) {
    "use strict";

    // update the camera
    if (this.surfaceId === undefined) {
        throw "surfaceId not defined";
    }
    var box = this.surface[this.surfaceId].box;

    this.camera.update(this.viewer.size, box, cam_transform);

    /*
      var gl = this.gl;
      // translate the model_matrix by the camera position
      var cam = this.camera.position;
    */
    /*
      xmat4.identity(this.model_matrix);
      //mat4.translate(this.model_matrix, [-cam[0], -cam[1], -cam[2]- 2.0*this.animator.zoom]);
      //xmat4.translate(this.model_matrix, this.model_matrix, [-cam[0], -cam[1], -cam[2]- 2.0*this.animator.zoom]);
      xmat4.translate(this.model_matrix, this.model_matrix, [cam[0], cam[1], -cam[2]- 2.0*this.animator.zoom]);
    */


    // FIX - creating temporary
    var tmp = xmat4.create();
    //mat4.multiply( this.camera.transform, this.model_matrix, tmp);
    //xmat4.multiply( tmp, this.camera.transform, this.model_matrix);
    //xmat4.translate(tmp, this.camera.transform, [cam[0], cam[1], -cam[2]- 2.0*this.animator.zoom] );
    for (var i in this.multishader.subshader()) {
        this.multishader.subshader()[i].setProjectiveTransform( this.camera.transform );
    }
};

Viewer.prototype._set_shader_parameters = function(obj_transform) {
    "use strict";

    if (this.surfaceId === undefined) {
        throw "surfaceId not defined";
    }

    var p = {};

    p.grid_frequency = this.surface[this.surfaceId].uvFrequency;
    p.stripe_frequency = this.surface[this.surfaceId].uvFrequency;
    p.lattice_frequency = this.surface[this.surfaceId].uvFrequency;


    // clipplane
    p.clip = false;
    p.clipplane = [0, 0, 0, 0];
    if (this.multishader.get_clipplane())
    {
        p.clip = true;
        var v = [0.0, 0.0, -1.0, 0.0];
        xmat4.multiply( v, obj_transform, v );
        p.clipplane = [v[0],v[1],v[2],v[3]];
    }

    // camera position
    // requires _set_camera() to be called first
    p.camera_position = this.camera.position;

    if (this.multishader.get_pline()) {
        this.pline_position[0] += this.pline_speed[0];
        this.pline_position[1] += this.pline_speed[1];
    }
    p.pline_position = clone( this.pline_position );

    this.multishader.setParameters(p);
};

/*
  Note on transforms:
  perspective_matrix is the camera
  model_matrix is the fixed position of the model
  view_matrix is the mouse transformation

  Total transform = perspective_matrix * model_matrix * view_matrix
*/
Viewer.prototype.draw = function(gl) {
    "use strict";

    if ( ! this.multishader)
    {
        return;
    }

    // load first surface
    if ( this.surfaceId === undefined)
    {
        var id = this.initial_surface;
        if ( this.surface[id] )
        {
            this.surface[id].load(this.gl);
            this.surfaceId = id;

            // set R3/S3 shader
            var s3 = this.surface[this.surfaceId].s3;
            this.multishader.setStereographicProjection(s3);
        }
        else
        {
            return;
        }
    }

    if ( ! gl ) {
        throw "viewer: gl is undefined";
    }


    if ( CONFIG.fps_counter )
    {
        // update the frames-per-second counter
        var fps = this.fpscounter.update();
        if (fps !== undefined)
        {
            this.controls.fps.textContent = fps.toFixed(2) + ' fps';
        }
    }

    // activate cubemap texture unit
    switch (this.multishader.getColorMode())
    {
    case COLORMODE_TEXIMAGE:
        if (this.teximage[this.surfaceId]) {
            gl.activeTexture(gl.TEXTURE0 + TEXIMAGE_TEXTURE_UNIT);
            gl.bindTexture(gl.TEXTURE_2D, this.teximage[this.surfaceId] );
        }
        break;
    case COLORMODE_CUBEMAP:
        if (this.cubemap) {
            gl.activeTexture(gl.TEXTURE0 + CUBEMAP_TEXTURE_UNIT);
            gl.bindTexture(gl.TEXTURE_CUBE_MAP, this.cubemap );
        }
        break;
    }

    var T = this.animator.get_transforms();
    var obj_transform = T[0];
    var cam_transform = T[1];

    // set camera
    this._set_camera(cam_transform);

    // set shader parameters
    this._set_shader_parameters(obj_transform);

    // draw surface
    //var metadata = surfaceMetadata[this.surfaceId];
    var S = this.surface[this.surfaceId];
    var p, q;
    if ('parts' in S) {
        p = S.parts[this.surface_part];
        q = S.remaining_parts;
    }

    //var parts = [ [0], []];
    //if (metadata.parts) {
    //  parts = [metadata.parts[this.surface_part], metadata.remaining_parts[this.surface_part]];
    //}

    this.surface[this.surfaceId].draw(
        gl,
        this.multishader,
        obj_transform,
        this.surface_part );


    gl.useProgram(null);
};

Viewer.prototype._load_surfaces = function() {
    "use strict";
    // initial surface
    this._load_surface(this.initial_surface);

    // remaining surfaces
    for (var i = 0; i < surface_data.length; ++i) {
        if (i == this.initial_surface) { continue; }
        this._load_surface(i);
    }
};

Viewer.prototype._load_surface = function(i) {
    "use strict";
    load_surface(
        //this.surface_list[i].surface,
        surface_data[i].path,
        function(this0,i0) {
            return function(surface) {
                this0.surface[i0] = surface;
            };
        }(this,i)
    );
};

/*
  -----------------------------------------------------------------------------
  load
  -----------------------------------------------------------------------------
*/
Viewer.prototype.load_teximage = function()
{
    "use strict";
    this.teximage = [null, null, null, null, null, null];

    function capture(this0, n0) {
        return function(texture){ this0.teximage[n0] = texture; };
    }


    for (var i = 0; i < surface_data.length; ++i) {
        XLAB.load_texture(
            this.gl,
            //url[i],
            baseURL + surface_data[i].texture,
            capture(this, i)
        );
    }
};

Viewer.prototype.load_cubemap = function()
{
    var texture_urls = [
        baseURL + "texture/x1.png",
        baseURL + "texture/x0.png",
        baseURL + "texture/y1.png",
        baseURL + "texture/y0.png",
        baseURL + "texture/z1.png",
        baseURL + "texture/z0.png"
    ];

    XLAB.load_cubemap(
        this.gl,
        texture_urls,
        function(this0) {
            return function(cubemap){ this0.cubemap = cubemap; }; }(this)
    );
};


Viewer.prototype.load_shader = function()
{
    "use strict";
    // load external shaders

    var urls = [baseURL + 'shader/multishader_vert.glsl',
                baseURL + 'shader/multishader_frag.glsl'];
    
    XLAB.load_files(urls,
                    function(this0, gl, extensions)
                    {
                        return function(src, urls) {
                            this0.multishader = new Multishader(src, gl, extensions, urls); };
                    }(this, this.gl, this.extensions)
                   );

};

Viewer.prototype.set_parts = function(n0)
{
    "use strict";
    var n = parse_int(n0);
    if (this.surface[this.surfaceId].parts)
    {
        this.multishader.set_texclip(0);
        this.surface_part = n;
    }
    else
    {
        // 0 = no clipping
        this.multishader.set_texclip( n == 2 ? 0 : n+1);
    }
    if ( CONFIG.triangle_counter ) {
        this.update_triangle_count();
    }
};


Viewer.prototype.update_triangle_count = function()
{
    "use strict";
    // -------------------------------------
    // triangle count
    var s = this.surface[this.surfaceId];
    var n = s.triangle_count;
    var p = 1;
    if ('parts' in s) {
        if (this.multishader.get_pline())
        {
            // drawing whole surface
            p = s.parts[2].length;
        }
        else
        {
            // drawing part of surface
            p = s.parts[this.surface_part].length;
        }
    }
    this.controls.triangle_count.textContent = format_sci(n*p) + ' triangles';
};


Viewer.prototype.set_surface = function(id)
{
    "use strict";
    if (id == this.surfaceId) {
        return;
    }

    // save old id
    var old_id = this.surfaceId;

    // unload old surface
    this.surface[old_id].unload(this.gl);

    // load the new surface
    this.surface[id].load(this.gl);
    // set the new id
    this.surfaceId = id;


    // -------------------------------------
    // set parts
    this.set_parts(2); // this has the side effect of setting the triangle counter

    // -------------------------------------
    // s3 shader
    var s3 = this.surface[this.surfaceId].s3;
    this.multishader.setStereographicProjection(s3);

    this.animator.clear();
};

Viewer.prototype._setup_body = function() {
    document.body.style.backgroundColor = 'black';
    document.body.style.color = 'white';
    document.body.style.margin = '0px';
    document.body.style.overflow = 'hidden';
};

Viewer.prototype.load_controls = function()
{
    "use strict";
    this.controls = new TouchControls( this );

    // resize callback
    function capture(this0) {
        return function(event) {
            this0.controls.on_resize(event);
        };
    }
    window.onresize = capture(this);
};


document.body.onload = function() {
    new Viewer();
};
