
/*global
  xmat4:false
*/

XLAB.Camera = function()
{
    "use strict";
    this.halfangle = 12.0;
    this.transform = xmat4.create();
    xmat4.identity(this.transform);
};

XLAB.Camera.prototype.update = function(panel_size, box, transformx)
{
    "use strict";
    //this.position = this._compute_camera_position(box);
    //mat4.perspective(2.0*this.halfangle, panel_size[0] / panel_size[1], this.clip[0], this.clip[1], this.transform);
    var rad = Math.PI / 180.0;
    var view_direction = [Math.cos(this.halfangle * rad), Math.sin(this.halfangle * rad)];

    this.camera_transform( this.transform, box, panel_size, view_direction);

    xmat4.multiply(this.transform, this.transform, transformx);

    this._compute_camera_position();
};

XLAB.Camera.prototype._compute_camera_position = function()
{
    "use strict";
    var m = this.transform;
    var a00 = m[ 0];
    var a03 = m[12];
    var a11 = m[ 5];
    var a13 = m[13];
    var a32 = m[11];
    var a33 = m[15];
    var p = [a03 * a32 / a00, a13 * a32 / a11, a33, -a32 ];
    this.position = [p[0]/p[3], p[1]/p[3], p[2]/p[3]];
};

/**
   Computes camera transform in a.
*/
XLAB.Camera.prototype._camera_transform = function( a, center, z_position, size, cutoff)
{
    "use strict";
    var x = center[0], y = center[1], z = center[2];
    var z0 = cutoff[0], z1 = cutoff[1];
    var zu = z_position[0], zv = z_position[1];
    var dx = size[0], dy = size[1];
    
    var m = zu - zv*z;

    // first row
    a[ 0] = m / dx;
    a[ 4] = 0.0;
    a[ 8] = 0.0;
    a[12] = -m * x / dx;

    // second row
    a[ 1] = 0.0;
    a[ 5] = m / dy;
    a[ 9] = 0.0;
    a[13] = -m * y / dy;

    // third row
    a[ 2] = 0.0;
    a[ 6] = 0.0;
    a[10] = (2.0*zu - zv*(z1+z0)) / (z1-z0);
    a[14] = (2.0*zv*z0*z1 - zu*(z0+z1)) / (z1-z0);

    // fourth row
    a[ 3] = 0.0;
    a[ 7] = 0.0;
    a[11] = -zv;
    a[15] = zu;
};

XLAB.Camera.prototype.camera_transform = function(a, box, aspect, view_direction)
{
    "use strict";
    // view_direction must be normalized
    //s = Math.sqrt(view_direction[0]*view_direction[0] + view_direction[1]*view_direction[1]);
    //v = [view_direction[0]/s, view_direction[1]/s];
    var v = view_direction;

    var xmin = box[0], ymin = box[1], zmin = box[2];
    var xmax = box[3], ymax = box[4], zmax = box[5];
    var radius = [ 0.5*(xmax-xmin), 0.5*(ymax-ymin), 0.5*(zmax-zmin) ];
    var center = [ 0.5*(xmax+xmin), 0.5*(ymax+ymin), 0.5*(zmax+zmin) ];

    /*
      Compute dx and dy.
    */
    var dx, dy;
    var zu, zv;

    if (aspect[0]*radius[1] >= aspect[1]*radius[0])
    {
        // wide window
        dy = radius[1];
        dx = (aspect[0]/aspect[1]) * dy;
        
        zu = dy*v[0] + center[2]*v[1];
        zv = v[1];
    }
    else
    {
        // tall window
        dx = radius[0];
        dy = (aspect[1]/aspect[0]) * dx;

        zu = dx*v[0] + center[2]*v[1];
        zv = v[1];
    }

    /*
      Compute z cutoffs.
    */
    var VIEWANGLE_CUTOFF = 1.0e-6;
    var ORTHOGONAL_CUTOFF = [ 1.0e3, -1.0e3 ];
    var PERSPECTIVE_CUTOFF_SCALE = [ 1.0e-3, 1.0e3 ];

    var ortho = v[1] < VIEWANGLE_CUTOFF;
    var cutoff = [];
    
    if (ortho)
    {
        // orthgraphic camera
        cutoff[0] = center[2] + ORTHOGONAL_CUTOFF[0];
        cutoff[1] = center[2] + ORTHOGONAL_CUTOFF[1];
    }
    else
    {
        // perspective cammera
        var z = zu/zv;
        cutoff[0] = z - (z-center[2]) * PERSPECTIVE_CUTOFF_SCALE[0];
        cutoff[1] = z - (z-center[2]) * PERSPECTIVE_CUTOFF_SCALE[1];
    }

    // camera transform without rotation
    //C = xmat4.create();
    this._camera_transform(a, center, [zu, zv], [dx, dy], cutoff );
    
    // camera rotation
    //  R = xmat4.create();
    //  this.rotationZ(R, rotation_angle);
    // product
    //  xmat4.multiply(a, C, R);
};
