/*global
  console:false
  XMLHttpRequest:false
  XLAB:false
*/

/*
  Loads the URL asynchronously and calls the callback with data as argument.
*/
XLAB.load_file = function(url, callback) {
    "use strict";
    var request = new XMLHttpRequest();
    request.open('GET', url);
    request.onreadystatechange = function(e) {
        if (request.readyState == 4) {
            callback(request.responseText);
        }
    };
    request.send();
};


/*
  XLAB.load_files(gl, urls, callback)
  Loads, compiles and links a vertex and fragment shader.

  gl   : the opengl context
  urls : array [vertexShaderSource, fragmentShaderSource]
  callback : called when shader is read and loaded into opengl.
  callback is of the form
  callback(shaderProgram)
  where shaderProgram is the newly created shader program.
*/

XLAB.load_files = function(urls, callback) {
    "use strict";
    var numUrls = urls.length;
    var numComplete = 0;
    var result = [];

    // Callback for a single file
    function partialCallback(text, i) {
        result[i] = text;
        numComplete++;

        // When all files have downloaded
        if (numComplete == numUrls) {
            callback(result, urls);
        }
    }

    function capture(i0) {
        return function(text) { partialCallback(text, i0); };
    }

    for (var i = 0; i < numUrls; i++) {
        XLAB.load_file( urls[i], capture(i) );
    }
    /*
      for (var i = 0; i < numUrls; i++) {
      XLAB.load_file(urls[i],
      function(i0) {
      return function(text) { partialCallback(text, i0); };
      }(i)
      );
      }
    */
};

/*
  callback:
  callback(text)
*/
XLAB.load_binary_file = function(url, callback) {
    "use strict";
    var request = new XMLHttpRequest();
    request.open('GET', url);
    request.responseType = 'arraybuffer';
    
    request.onload = function(e) {
        if (request.readyState == 4) {
            callback(request.response);
        }
    };
    request.send();
};

/*
  callback:
  callback(result, urls)
*/
XLAB.load_binary_files = function(urls, callback) {
    "use strict";
    var numUrls = urls.length;
    var numComplete = 0;
    var result = [];

    // Callback for a single file
    function partialCallback(text, i) {
        result[i] = text;
        numComplete++;

        // When all files have downloaded
        if (numComplete == numUrls) {
            callback(result, urls);
        }
    }

    function capture(i0) {
        return function(text) {partialCallback(text,i0);};
    }

    for (var i = 0; i < numUrls; i++) {
        XLAB.load_binary_file( urls[i], capture(i) );
    }
/*
    for (var i = 0; i < numUrls; i++) {
        XLAB.load_binary_file(
            urls[i],
            function(i0) {
                return function(text) {partialCallback(text,i0);};
            }(i)
        );
    }
*/
};
