/*global
  XLAB:false
*/

/*
 ShaderProgram
*/
XLAB.ShaderProgram = function() {
    "use strict";
};


/*
  Links already-compiled shaders.
*/
XLAB.ShaderProgram.prototype.link = function(gl, shaders) {
  "use strict";
  if (this.program) {
    gl.deleteProgram(this.program);
  }

  // save shaders for deletion
  this.shaders = shaders;
  this.program = gl.createProgram();

  var urls = [];
  for (var i in shaders) {
    gl.attachShader(this.program, shaders[i].shader);
    urls.push(shaders[i].url);
  }

  gl.linkProgram(this.program);

  // check error
  if ( ! gl.getProgramParameter(this.program, gl.LINK_STATUS) ) {
    throw 'failed to link shaders urls=' + urls;
  }
};

XLAB.ShaderProgram.prototype.use = function(gl) {
  "use strict";
  if ( ! gl ) {
    throw "shader: gl is undefined";
  }
  gl.useProgram(this.program);
};

XLAB.ShaderProgram.prototype.getAttribLocation = function(gl,key) {
  "use strict";
  return gl.getAttribLocation(this.program, key);
};

XLAB.ShaderProgram.prototype.getUniformLocation = function(gl,key) {
  "use strict";
  return gl.getUniformLocation(this.program, key);
};

XLAB.ShaderProgram.prototype.destroy = function(gl) {
  "use strict";
  var s;
  for (s in this.shaders) {
    gl.detachShader(this.program, this.shaders[s].shader);
  }
  for (s in this.shaders) {
    gl.deleteShader(this.shaders[s].shader);
  }
  gl.deleteProgram(this.program);
};

/**
  Helper function to compile and link a shader from src.
*/
XLAB.createShader = function(gl, src, urls) {
  "use strict";

  var vertex_shader = new XLAB.Shader(gl.VERTEX_SHADER);
  vertex_shader.compile(gl, src[0], urls ? urls[0] : '<unknown>');

  var fragment_shader = new XLAB.Shader(gl.FRAGMENT_SHADER);
  fragment_shader.compile(gl, src[1], urls ? urls[1] : '<unknown>' );
  
  var shaderProgram = new XLAB.ShaderProgram();
  shaderProgram.link(gl, [vertex_shader, fragment_shader] );

  return shaderProgram;
};
