/*global
  xmat4:false
  get_time:false
  document:false
  console:false
*/




function clamp(x, a, b) {
    "use strict";
    if (x < a) { x = a; }
    if (x > b) { x = b; }
    return x;
}

function abs2(x, y) {
    "use strict";
    return Math.sqrt(x*x + y*y);
}

function abs3(x, y, z) {
    "use strict";
    return Math.sqrt(x*x + y*y + z*z);
}


XLAB.Touch = function(animator, canvas) {
    "use strict";
    this.animator = animator;
    this.connect(canvas);
    //this.time = 0;
    //this.fixed_time_interval = 1000;

    this.current_zoom = 0.0;
    this.blocked = false;
    this.count = 0;
};

function dump_touch(label, event) {
    "use strict";
/*
    var s = "";
    for (var i = 0; i < event.targetTouches.length; ++i) { 
        s += "(" + event.targetTouches[i].pageX + " " + event.targetTouches[i].pageY + ")";
    }
    console.log(label + " " + event.targetTouches.length + " " + s);
*/
}

/*
  this.identifier maps (touch identifier) -> index, where
  index is an integer 0,1,.. up to the number of active touches.

  This map is assigned in two situations:
  -- ontouchstart
  -- ontouchmove if the number of touch points has changed
*/


// start
XLAB.Touch.prototype._onTouchStart = function(event) {
    "use strict";

    this.blocked = false;

    dump_touch("START", event);
    //this.time = get_time();

    event.stopPropagation();
    event.preventDefault();

    this.count = event.targetTouches.length;

    // record identifiers
    this.identifier = [];
    var a = [];
    for (var i=0; i< event.targetTouches.length; ++i) {
        // this.identifier is a map mapping identifier -> index
        this.identifier[ event.targetTouches[i].identifier ] = i;
        a.push( event.targetTouches[i].pageX );
        a.push( event.targetTouches[i].pageY );
    }

    // clear stack and push the points
    this.start(a);
    this.stack = [];
    this.stack.push(a);

    return false;
};


// move
XLAB.Touch.prototype._onTouchMove = function(event) {
    "use strict";

    dump_touch("MOVE", event);

    event.stopPropagation();
    event.preventDefault();

    if (this.blocked) { return; }

    if( this.count == event.targetTouches.length ) {
        //this.time = get_time();
        // sort coordinates to same order as start
        var a = [];
        for (var i=0; i< event.targetTouches.length; ++i) {
            // get identifier
            var id = event.targetTouches[i].identifier;
            // lookup index from identifer
            var m = this.identifier[id];
            a.push(event.targetTouches[m].pageX);
            a.push(event.targetTouches[m].pageY);
        }

        this.move(this.stack[0], a);
        // push a onto the stack
        this.stack.push(a);
        while (this.stack.length > 4) {
            this.stack.shift();
        }
    }
    else {

        // the number of fingers changed
        // ignore if this happened in a short time interval

        /*
          === LAG ===
          t = get_time();
          if (t < this.time + this.fixed_time_interval) {
          this.animator.lag();
          return;
          }
          this.time = get_time();
        */
        // stop previous motion
        this.end(this.count, event.targetTouches.length);
        this.blocked = true;

/*
        // restart with new count
        this._onTouchStart(event);
*/
    }


    return false;
};


// end
XLAB.Touch.prototype._onTouchEnd = function(event) {
    "use strict";

    dump_touch("END", event);
    event.stopPropagation();
    event.preventDefault();

    if (this.blocked) { return; }

    //this.end(this.count, event.targetTouches.length);
    this.end(this.count, this.count);
    this.blocked = true;

    //this.end(this.count, event.targetTouches.length);

    return false;
};

// cancel
XLAB.Touch.prototype._onTouchCancel = function(event) {
    "use strict";
    var s = "";
    for (var i = 0; i < event.targetTouches.length; ++i) { 
        s += "(" + event.targetTouches[i].pageX + " " + event.targetTouches[i].pageY + ")";
    }

    event.stopPropagation();
    event.preventDefault();

    this.end(this.count, event.targetTouches.length);

    return false;
};





XLAB.Touch.prototype.start = function(a) {
    "use strict";
    switch(a.length/2) {
    case 0:
        break;
    case 1:
        this.animator.rotateR3_start(
            [a[0], a[1]]);
        break;
    case 2:
        this.animator.twofinger_start(
            [a[0], a[1]], [a[2], a[3]]);
        break;
    default:
        var x = (a[0] + a[2] + a[4])/3.0;
        var y = (a[1] + a[3] + a[5])/3.0;
        this.animator.rotateS3_start([x,y]);
        break;
    }
};





XLAB.Touch.prototype.move = function(a, b) {
    "use strict";

    switch(a.length/2) {
    case 0:
        break;
    case 1:
        this.animator.rotateR3_move(
            [a[0], a[1]], [b[0], b[1]]);
        break;
    case 2:
        //this.animator.twofinger_move(
        //[a[0], a[1]], [a[2], a[3]], [b[0], b[1]], [b[2], b[3]]);

        this.move2(
            [a[0], a[1]], [a[2], a[3]], [b[0], b[1]], [b[2], b[3]]);
        break;
    default:
        // p = average of a's
        // q = average of b's
        var p = [ (a[0] + a[2] + a[4])/3.0, (a[1] + a[3] + a[5])/3.0 ];
        var q = [ (b[0] + b[2] + b[4])/3.0, (b[1] + b[3] + b[5])/3.0 ];
        this.animator.rotateS3_move(p, q);
        break;
    }
};

// change from m fingers to n fingers
XLAB.Touch.prototype.end = function(m, n) {
    "use strict";
    //if (n !== 0) {
    //    return;
   // }
    switch (m) {
    case 0:
        break;
    case 1:
        this.animator.rotateR3_end();
        break;
    case 2:
        this.animator.twofinger_end();
        break;
    default:
        this.animator.rotateS3_end();
        break;
    }
};

XLAB.Touch.prototype.move2 = function(a0, b0, a1, b1) {
    "use strict";

    this.rotation_speed = 0.4;
    this.translation_speed = 0.0013;
    this.zoom_speed = 0.01;

    // previous length
    var dx = a0[0]-b0[0];
    var dy = a0[1]-b0[1];
    var d0 = abs2(dx, dy);

    // current length
    dx = a1[0]-b1[0];
    dy = a1[1]-b1[1];
    var d1 = abs2(dx, dy);

    //var speed = (this.current_zoom + 30)/300.0;
    //speed = clamp(speed, 0.01, 1.0);

    //var speed = Math.exp(clamp(-0.1*this.current_zoom, -4.0, 4.0));
    var zoom = this.zoom_speed * (d0-d1);
    //zoom = clamp( zoom, -4.0, 4.0 );

    //----------------
    // rotation angle (note atan2(y,x) is backwards order)
    var angle0 = Math.atan2(b0[1]-a0[1], b0[0]-a0[0]);
    var angle1 = Math.atan2(b1[1]-a1[1], b1[0]-a1[0]);
    var angle = -(angle1 - angle0);

    //angle *= this.rotation_speed;
    var pi = Math.PI;
    var twopi = 2.0*Math.PI;
    if (angle > pi) {
        angle -= twopi;
    }
    if (angle < -pi) {
        angle += twopi;
    }
    angle *= this.rotation_speed;

    //----------------
    // translation
    // compute average position of the two fingers
    var x0 = 0.5*(a0[0] + b0[0]);
    var y0 = 0.5*(a0[1] + b0[1]);
    var x1 = 0.5*(a1[0] + b1[0]);
    var y1 = 0.5*(a1[1] + b1[1]);

    //speed = Math.exp(clamp(-0.01*this.current_zoom, -4.0, 4.0));
    var s = this.translation_speed; // * speed;
    var t = [s*(x1-x0), -s*(y1-y0)];
    //t[0] = clamp(t[0], -4.0, 4.0);
    //t[1] = clamp(t[1], -4.0, 4.0);

    var zoom_m = Math.abs( this.zoom_speed * (d0-d1) );
    var angle_m = 4.0*Math.abs(angle);
    var trans_m = 2.0*abs2(t[0], t[1]);

    //this.zoom_cutoff = [-40.0, 4.0];


    // pick one
    if (zoom_m >= angle_m && zoom_m >= trans_m) {
        //var q = this.current_zoom - zoom;
        //    if (q >= this.zoom_cutoff[0] && q <= this.zoom_cutoff[1]) {
        //        this.current_zoom -= zoom;
        //        this.animator.translateZ( zoom );
        //    }
        //    this.current_zoom -= zoom;
        this.animator.translateZ( zoom );
    }
    else if (angle_m >= zoom_m && angle_m >= trans_m ) {
        this.animator.rotateZ( angle );
    }
    else if (trans_m >= zoom_m && trans_m >= angle_m ) {
        this.animator.translateXY( t );
    }

    /*
    // all three

    this.animator.translateZ( zoom );
    this.animator.translateXY( t );
    this.animator.rotateZ( angle );
    */
};

XLAB.Touch.prototype.connect = function(canvas) {
    "use strict";

    // events are added to the document, not to the canvas
    // this is wrong if the canvas is not fullscreen
    // this is so that events on overlapping divs without callbacks
    // propogate
    // this is necessary because children of a canvas do not show
    // (childern of canvas are meant as default when canvas is not supported

    document.addEventListener(
        'touchstart',
        function(this0) {
            return function(event) {
                this0._onTouchStart(event); }; }(this),
        false
    );

    document.addEventListener(
        'touchmove',
        function(this0) {
            return function(event) {
                this0._onTouchMove(event); }; }(this),
        false
    );
    document.addEventListener(
        'touchend',
        function(this0) {
            return function(event) {
                this0._onTouchEnd(event); };  }(this),
        false
    );
    document.addEventListener(
        'touchcancel',
        function(this0) {
            return function(event) {
                this0._onTouchCancel(event); };  }(this),
        false
    );
};
