/*global
  WebGLDebugUtils:false
  console:false
  window:false
  document:false
  WebGLRenderingContext:false
  requestAnimFrame:false
*/

XLAB.Viewer = function(parent, p) {
    "use strict";
    this.canvas = this._create_canvas();
    
    parent.appendChild(this.canvas);
    this.gl = this._initGL(this.canvas, p);
};

XLAB.Viewer.prototype.run = function(callback)
{
    "use strict";
    this.callback = callback;
    this._draw();
};

XLAB.Viewer.prototype._set_viewport = function()
{
    "use strict";
    // set canvas size to fill the browser window
    this.size = [window.innerWidth, window.innerHeight];
    this.canvas.width = this.size[0];
    this.canvas.height = this.size[1];

    this.gl.viewport(0, 0, this.size[0], this.size[1]);
};

XLAB.Viewer.prototype._clear = function()
{
    "use strict";
    this.gl.clear(
        WebGLRenderingContext.COLOR_BUFFER_BIT |
            WebGLRenderingContext.DEPTH_BUFFER_BIT);
};

XLAB.Viewer.prototype._draw = function()
{
    "use strict";
    requestAnimFrame(
        function(this0)
        {
            return function()
            {
                this0._draw();
            };
        }(this) );

    this._set_viewport();
    this._clear();
    this.callback(this.gl);
};


XLAB.Viewer.prototype._create_canvas = function()
{
    "use strict";
    try
    {
        var canvas = document.createElement('canvas');

        //    canvas.style.position = 'fixed';
        //canvas.style.zIndex = 0;
        //canvas.border = '0px solid black';
        
        if (! canvas) {
            throw 'Could not create canvas';
        }
        return canvas;
    } catch (error) {
        console.error(error);
    }
};

XLAB.Viewer.prototype._initGL = function(canvas, p)
{
    "use strict";
    var antialias = 'antialias' in p ? p.antialias : false;
    var debug = 'debug' in p ? p.debug : false;
    var gl;

    var params = { antialias : antialias };
    gl = canvas.getContext("experimental-webgl", params);

    if ( ! gl ) {
        throw 'Could not create WebGL context';
    }
    if (debug) {
        gl = WebGLDebugUtils.makeDebugContext(gl);
    }
    if ( ! gl ) {
        throw 'Could not create WebGL debug context';
    }
    return gl;
};
