// FRAGMENT SHADER

#if defined(have_OES_standard_derivatives)
#extension GL_OES_standard_derivatives : enable
#endif


#if defined(PATTERN_SQUARE) || defined(PATTERN_CIRCLE) || defined(PATTERN_SUPERELLIPSE) || defined(PATTERN_STRIPE0) || defined(PATTERN_STRIPE1)
#define PATTERN
#endif

#if defined(PLINE_FULL) || defined(PLINE_PART)
#define PLINE
#endif

#if defined(PATTERN) || defined(PLINE) || defined(TEXIMAGE) || defined(TEXCLIP_X) || defined(TEXCLIP_Y)
#define TEXTURE
#endif

//#if defined(SPECULAR) || defined(CLIP) || defined(CUBEMAP)
#if defined(SPECULAR) || defined(CUBEMAP)
#define POSITION
#endif


#if defined(PATTERN) && !defined(CUTAWAY)
#define GRID
#endif


precision mediump float;

#if defined(POSITION)
varying vec4 position_i;
#endif
varying vec3 normal_i;
#if defined(TEXTURE)
varying vec2 texture_i;
varying vec2 texture_it;
#endif
#if defined(PLINE)
varying vec2 texture_j;
#endif
#if defined(COLOR)
varying float color_i;
#endif

uniform vec3 light_direction;
uniform vec3 ambient;
uniform vec3 diffuse_front;
uniform vec3 diffuse_back;
#if defined(SPECULAR)
uniform vec3 specular;
#endif
uniform float shininess;
uniform vec3 camera_position;

#if defined(PATTERN)
uniform float pattern_width;
uniform vec2 pattern_exponent;
uniform vec3 pattern_front_color;
uniform vec3 pattern_back_color;
//uniform float stripe_width;
#endif

#if defined(CLIP)
//uniform vec4 clipplane;
varying float clip_distance;
#endif

#if defined(PLINE)
uniform vec3 pline_x_color;
uniform vec3 pline_y_color;
uniform vec2 pline_position;
uniform vec2 pline_width;
#endif

#if defined(TEXIMAGE)
uniform sampler2D teximage;
#endif
#if defined(CUBEMAP)
uniform samplerCube cubemap;
#endif

#if defined(LOCALCOLOR)
uniform vec3 localcolor0;
uniform vec3 localcolor1;
#endif


/*
-------------------------------------------------------------
helper functions
-------------------------------------------------------------
*/
float diffuse_factor(in float t)
{
  return 0.25 * (1.0+t) * (1.0+t);
}

#if defined(SPECULAR)
// specular (gaussian)
float specular_factor(in float t, in float shininess)
{
  float u = acos(t)/shininess;
  return exp(-u*u);
}
#endif

vec3 light( vec3 color, vec3 normal )
{
  vec3 N = normalize(normal);
  vec3 L = normalize(light_direction);

  // ambient + diffuse    
  float t = dot(N, L);
  vec3 C = ambient + color * diffuse_factor( t );

#if defined(SPECULAR)
  // specular
  vec3 V = normalize(camera_position - position_i.xyz);
  vec3 R = reflect(-L, N);
  C += specular * specular_factor(dot(R,V), shininess);
#endif

  return C;
}

// ------------
// grid
// ------------

#if defined(PATTERN) && !defined(CUTAWAY)
float smooth(float x)
{

#if 0
#if defined(have_OES_standard_derivatives)
  float d = 0.2*(fwidth(texture_it.x) + fwidth(texture_it.y));
  return smoothstep(pattern_width-d, pattern_width+d, x);
#else
  return step(pattern_width, x);
#endif
#endif
  return step(pattern_width, x);
}
#endif

#if defined(PATTERN_SQUARE)
// square grid
float pattern()
{
  vec2 q = abs(2.0 * fract(texture_it) - 1.0);
  return max(q[0],q[1]);
}
#elif defined(PATTERN_CIRCLE)
float pattern()
{
  vec2 q = abs(2.0 * fract(texture_it) - 1.0);
  return dot(q,q);
}
#elif defined(PATTERN_SUPERELLIPSE)
float pattern()
{
  vec2 q = abs(2.0 * fract(texture_it) - 1.0);
  q = pow(q, vec2(2.0, 2.0));
  return dot(q,q);
}
#elif defined(PATTERN_STRIPE0)
float pattern()
{
  float q = abs(2.0 * fract(texture_it.x) - 1.0);
  return q;
}
#elif defined(PATTERN_STRIPE1)
float pattern()
{
  float q = abs(2.0 * fract(texture_it.y) - 1.0);
  return q;
}
#endif

// ------------
// pline
// ------------

#if defined(PLINE)
// for parameter lines
// see Wikipedia, Smoothstep
float smootherstep(in float edge0, in float edge1, in float x)
{
  float t = clamp((x - edge0)/(edge1 - edge0), 0.0, 1.0);
  return t*t*t*(t*(t*6.0 - 15.0) + 10.0);
}

// for parameter lines
// see http://www.yaldex.com/open-gl/ch17lev1sec4.html
vec2 pline_factor()
{
  float FWIDTH_SCALE = 1.5;

  // triangle going from (0,1) to (1/2,0) to (1,1)
  vec2 x = abs(2.0 * fract(texture_j.xy - pline_position) - 1.0);

  vec2 w = 1.0 - 0.5*pline_width;



  // TODO: check for OES_standard_derivatives
//  float dx = FWIDTH_SCALE * fwidth(texture_j.x);
//  float dy = FWIDTH_SCALE * fwidth(texture_j.y);

//  return vec2( smootherstep(w.x-dx, w.x+dx, x.x), smootherstep(w.y-dy, w.y+dy, x.y) );
  return step(w, x);
}
#endif

#if defined(PLINE_FULL)
vec3
mix_pline(in vec3 color, in vec2 s)
{
  if (s.x > 0.0)
  {
    if (s.y > 0.0)
    {
      // x and y stripe
      return mix( pline_y_color, pline_x_color, s.x );
    }
    else
    {
      // x stripe
      return mix( color, pline_x_color, s.x );
    }
  }
  else if (s.y > 0.0)
  {
    // y stripe
    return mix( color, pline_y_color, s.y );
  }
  else
  {
    // no stripe
    return color;
  }
}
#endif
#if defined(PLINE_PART)
vec3
mix_pline(in vec2 s)
{
  if (s.y > 0.0)
  {
    return pline_y_color;
  }
  if (s.x > 0.0)
  {
    return pline_x_color;
  }
  return vec3(0.0, 0.0, 0.0);
}
#endif

void clip()
{
#if defined(CLIP)
  //if (dot(position_i,clipplane) < 0.0)
  if (clip_distance < 0.0)
  {
    discard;
  }
#endif

#if defined(TEXCLIP_X)
  if (texture_i.x > 0.5) { discard; }
#endif
#if defined(TEXCLIP_Y)
  if (texture_i.y > 0.5) { discard; }
#endif

#if defined(PATTERN) && defined(CUTAWAY)
  float q = pattern();
  if (q < 0.5) { discard; }
#endif
}


void
get_color(out vec3 front_color, out vec3 back_color)
{
#if defined(TEXIMAGE)
  vec4 color4 = texture2D(teximage, texture_i);
  front_color = color4.xyz;
  back_color = front_color;
#elif defined(CUBEMAP)
  vec3 V = position_i.xyz - camera_position*position_i.w;
  vec3 R = reflect( V, normalize(normal_i) );
  vec4 color4 = textureCube( cubemap, R );
  front_color = color4.xyz;
  back_color = front_color;
#elif defined(LOCALCOLOR)
  front_color = mix(localcolor0, localcolor1, color_i);
  back_color = front_color;
#else
  front_color = diffuse_front;
  back_color = diffuse_back;
#endif
}


void
apply_lighting(in vec3 front_color, in vec3 back_color)
{
  // apply lighting
  vec3 color;
  if (gl_FrontFacing)
  {
#if defined(GRID)
    front_color = mix( front_color, pattern_front_color, smooth(pattern()) );
#endif
#if defined(PLINE_FULL)
    front_color = mix_pline( front_color, pline_factor() );
#endif
    color = light(front_color, normal_i);
  }
  else
  {
#if defined(GRID)
    back_color = mix( back_color, pattern_back_color, smooth(pattern()) );
#endif
#if defined(PLINE_FULL)
    back_color = mix_pline( back_color, pline_factor() );
#endif
    color = light(back_color, -normal_i);
  }

  gl_FragColor = vec4(color, 1.0);
}

#if defined(PLINE_PART)
void
pline_part()
{
  vec2 p = pline_factor();
  if (p.x == 0.0 && p.y == 0.0) {
    discard;
  }
  vec3 color = mix_pline( p );
  if (gl_FrontFacing)
  {
    color = light(color, normal_i);
  }
  else
  {
    color = light(color, -normal_i);
  }
  gl_FragColor = vec4(color, 1.0);
}
#endif


/*
-------------------------------------------------------------
main
-------------------------------------------------------------
*/
void main(void)
{

#if defined(PLINE_PART)
  pline_part();
  return;
#endif

#if defined(PLINE_FULL)
  vec2 p = pline_factor();
  if (p.x == 0.0 && p.y == 0.0)
  {
    clip();
  }
#else
  clip();
#endif

  vec3 front_color;
  vec3 back_color;
  get_color(front_color, back_color);
  apply_lighting(front_color, back_color);
}
